dojo.require("dijit.Tree");
dojo.provide("custom.BrowseCrTree");

dojo.extend(dijit._TreeNode,
{	
	templateString: "<div dojoAttachPoint=\"treeNode\" class=\"dijitTreeNode\" role=\"presentation\">" +
		"<div dojoAttachPoint=\"rowNode\" class=\"dijitTreeRow dijitInline\" role=\"presentation\">" +
			"<div dojoAttachPoint=\"indentNode\" class=\"dijitInline\">" +
			"<span dojoAttachPoint=\"expandoNode\" class=\"dijitTreeExpando\"  role=\"presentation\"></span>" + 
			"<span dojoAttachPoint=\"expandoNodeText\" class=\"dijitExpandoText\" role=\"presentation\"></span>" + 
			"<div dojoAttachPoint=\"contentNode\" class=\"dijitTreeContent\" role=\"presentation\">" + 
			"<img dojoAttachPoint=\"iconNode\" class=\"\" style=\"display: none; width: 18px; height: 18px;\"></img>" +
			"<span dojoAttachPoint=\"labelNode\"\n\tdojoAttachEvent=\"onmouseover:_onLabelMouseOver, onmouseout:_onLabelMouseOut\"" + 
				"tabindex=\"-1\" class=\"dijitTreeLabel\" role=\"treeitem\" waiState=\"selected-false\" dojoAttachEvent=\"onfocus:_onNodeFocus\">" +
			"</span>" +
		"</div></div></div><div dojoAttachPoint=\"containerNode\" class=\"dijitTreeContainer\" role=\"presentation\"\"></div></div>",
	postCreate: function()
	{
		// set label, escaping special characters
		//this.setLabelNode(this.label);
		
		// Set the icon source for the node
		if (this.item.icon)
		{
			this.setIconNode(this.item.icon);
		}
	
		// set expand icon for leaf
		this._setExpando = this.setExpando;
		this.setExpando();
	
		// set icon and label class based on item
		this._updateItemClasses(this.item);
	
		if (this.isExpandable)
		{
			dijit.setWaiState(this.labelNode, "expanded", this.isExpanded);
		}
	},
	setExpando: function(/*Boolean*/ processing)
	{
		// summary: set the right image for the expando node
		// apply the appropriate class to the expando node
		var styles = ["dijitTreeExpandoLoading", "dijitTreeExpandoOpened",
			"dijitTreeExpandoClosed", "dijitTreeExpandoLeaf", "dijitTreeExpando"];
			
		var iconStyles = ["dijitInline", "dijitTreeIcon"];
		
		dojo.forEach(styles, function(s)
		{
			dojo.removeClass(this.expandoNode, s);
		}, this);
		
		if (this.item && !(this.item.noResult))
		{	
			dojo.addClass(this.expandoNode, styles[4]);
			dojo.addClass(this.expandoNode, styles[2]);		
		}
		else
		{
			dojo.removeClass(this.expandoNode, styles[4]);
			dojo.addClass(this.expandoNode, styles[3]);	
		}

		// provide a non-image based indicator for images-off mode
		this.expandoNodeText.innerHTML = processing
			? "*" 
			: this.isExpandable 
				? (this.isExpanded ? "-" : "+")
				: "*";
	},
	
	setIconNode: function(/*img source*/ icon)
	{
		this.iconNode.src = icon;
		dojo.style(this.iconNode, "display", "");
	},
	
	setSelectedNode: function(on)
	{
		if (on)
		{
			dojo.style(this.labelNode, "fontWeight", "bold");
		}
		else
		{
			dojo.style(this.labelNode, "fontWeight", "");
		}
	},
	
	_onLabelMouseOver: function(e)
	{
		var domElement = e.target;
		var nodeWidget = dijit.getEnclosingWidget(domElement); 
		var labelNode = nodeWidget.labelNode;
		
		if (!nodeWidget.item.noResult)
		{
			dojo.removeClass(labelNode, "dijitTreeLabel");
			dojo.addClass(labelNode, "labelNode");
		}
	},
	
	_onLabelMouseOut: function(e)
	{
		var domElement = e.target;
		var labelNode = dijit.getEnclosingWidget(domElement).labelNode;
		
		dojo.removeClass(labelNode, "labelNode");
	}
});
/*
 * Customized tree for browsing CRs.  
 */
dojo.declare("custom.BrowseCrTree", dijit.Tree,
{
	templateString: "<div class=\"dijitTree dijitTreeContainer\" role=\"tree\"\n\tdojoAttachEvent=\"onclick:onClick,onkeypress:_onKeyPress\"></div>\n",
	
	lastSelected : null,
	isIe6: (dojo.isIE && (navigator.userAgent.indexOf("MSIE 6.0") != -1)),
	
	_load: function(){
		// summary: initial load of the tree
		// load root node (possibly hidden) and it's children
		this.model.getRoot(
			dojo.hitch(this, function(item){
				var rn = this.rootNode = new dijit._TreeNode({
					item: item,
					tree: this,
					isExpandable: true,
					label: this.label || this.getLabel(item)
				});
				if(!this.showRoot){
					rn.rowNode.style.display="none";
				}							
				
				rn.labelNode.style.cursor = "default";
				rn.labelNode.style.textDecoration = "none";
				rn.labelNode.style.fontWeight = "bold";
								
				rn.expandoNode.className="";				
				
				this.domNode.appendChild(rn.domNode);
				this._itemNodesMap[this.model.getIdentity(item)] = rn;

				rn._updateLayout();		// sets "dijitTreeIsRoot" CSS classname

				// load top level children
				this._expandNode(rn);
				rn.expandoNode.className="";
			}),
			function(err){
				console.error(this, ": error loading root: ", err);
			}
		);
	},
	
	
	
	/*
	 * Remove all the style classes and add the classes given by the 
	 * parameter 'classNames'.
	 */			
	_removeAndAddClass: function(/*Widget*/ nodeWidget, /*Array*/ classNames)
	{
		var styles = ["dijitTreeNode", "dijitTreeExpando", "dijitTreeExpandoLoading", "dijitTreeExpandoOpened",
	  		"dijitTreeExpandoClosed", "dijitTreeIsLast", "dijitTreeExpandoLeaf"];

			// Remove all existing classes
		dojo.forEach(styles, function(style)
  		{
			dojo.removeClass(nodeWidget, style);
  		}, this);
  		
  		// Add classes.
  		dojo.forEach(classNames, function(className)
  		{
  			dojo.addClass(nodeWidget, className);
  		}, this);
	},
	
	
	/*
	 * Get only the required attributes from the item.
	 */
	_getObject: function(/*Item*/ item)
	{
		var store = this.tree.model.store;
		var attribs = store.getAttributes(item);
		var retObj = new Object();
		
		dojo.forEach(attribs, function(attrib)
		{
			retObj[attrib] = store.getValue(item, attrib);
		});
		
		return retObj; 
	},
	
	/*
	 * Handle click event.
	 */
	onClick: function(e)
	{
		var domElement = e.target;
		var nodeWidget = dijit.getEnclosingWidget(domElement);
		
		var item = this.model.store.isItem(nodeWidget.item)
			? this._getObject(nodeWidget.item)
			: nodeWidget.item;
		
		// Return if not clicked on Label.
		if (!nodeWidget || !nodeWidget.isTreeNode || item.noResult || nodeWidget.item.root)
		{
			return;
		}
		
		if ((domElement == nodeWidget.expandoNode || domElement == nodeWidget.expandoNodeText) && nodeWidget.expandoNode.className != "dijitTreeExpando dijitTreeExpandoLeaf")
		{
			
			this._removeAndAddClass(nodeWidget.expandoNode, ["dijitTreeExpando", "dijitTreeExpandoLoading"]);					
			
			if (nodeWidget.getChildren().length == 0)
			{	
				// Load the relation information by an ajax call.
				window.lookupRelationInformation(this, item, nodeWidget);
			}
			else
			{
				this.removeChildren(nodeWidget);
				this._removeAndAddClass(nodeWidget.expandoNode, ["dijitTreeExpando", "dijitTreeExpandoClosed"]);
			}
		}
		
		if (domElement == nodeWidget.labelNode)
		{
			switch (item.cvtype)
			{
			case "problem": 
				showChangeRequest(item);
				break;
			case "task": 
				showTask(item);
				break;					
			default:
				var isAttachment = isAttachmentObject(item);
				var isDoorsRequirement = isDoorsObject(item);
				
				//this must be checked first as these are also considered to be attachments.
				if (isDoorsRequirement)
				{
					showDoorsObject(item);
				}
				else if (isAttachment)
				{
					downloadAttachment(item);
				}
				else
				{
					showObject(item);
				}
			}
			if (this.lastSelected != null)
				this.lastSelected.setSelectedNode(false);
			this.lastSelected = nodeWidget;
			nodeWidget.setSelectedNode(true);
		}
		nodeWidget.containerNode.style.display="inline";
		dojo.stopEvent(e);
	},
	
	
	/*
	 * Overridden this method as customization is taking care of loading the 
	 * appropriate classes for icons. Need to return empty string as initial 
	 * loading of tree is automatic.
	 */
	getIconClass: function(/*dojo.data.Item*/ item, /*Boolean*/ opened)
	{
		return "";
	},	
	
	
	/*
	 * Update the tree with relation data.
	 */
	updateTree: function(treeNode, relations, results)
	{
		var foundResults = false;
		
		//loop over each of the expected relations.
		for (var i = 0; i < relations.length; i++)
		{
			var relationResults = results[relations[i]];
			
			var img = dojo.byId(relations[i] + "_img");				
			
			// Populate the tree nodes.
			if (relationResults != null && relationResults != undefined)
			{				
				//Loop over each result for the looked up relation.
				for (var j = 0; j < relationResults.length; j++)
				{
					var isAttachment = false;
					var isDoorsRequirement = false;
					var cvtype = relationResults[j][AttributeNames.CVTYPE];
					relationResults[j].icon = img.src;
					
					if ((cvtype != "problem") && (cvtype != "task"))
					{
						isAttachment = isAttachmentObject(relationResults[j]);
						isDoorsRequirement = isDoorsObject(relationResults[j]);
					}
					
					if (cvtype == "problem")
					{						
						var subTreeNode = new dijit._TreeNode(
						{
							item: relationResults[j],
							tree: treeNode.tree,
							isExpandable: true,
							label: relationResults[j].__title__
						});
						
						treeNode.addChild(subTreeNode);
						
						if (!this.isIe6 && (subTreeNode.getParent().getParent().id == "dijit__TreeNode_0"))
						{
							dojo.style(subTreeNode.rowNode.parentNode, "marginLeft", "40px");
						}
						else
						{
							if (subTreeNode.getParent().getParent().id == "dijit__TreeNode_0")
							{
								dojo.style(subTreeNode.rowNode.parentNode, "marginLeft", "40px");
							}
						}
						
						//donot expand if the the same CR is shown in the tree.
						if (treeNode.getParent().item.cvid == subTreeNode.item.cvid)
						{
							subTreeNode.isExpandable = false;						
							this._removeAndAddClass(subTreeNode.expandoNode, ["dijitTreeExpando", "dijitTreeExpandoLeaf"]);
						}
						
						foundResults = true;
					}
					else if (cvtype == "task")
					{						
						var subTreeNode = new dijit._TreeNode(
						{
							item: relationResults[j],
							tree: treeNode.tree,
							isExpandable: true,
							label: relationResults[j].__title__ 
						});
						
						treeNode.addChild(subTreeNode);
						if (!this.isIe6 && (subTreeNode.getParent().getParent().id == "dijit__TreeNode_0"))
						{
							dojo.style(subTreeNode.rowNode.parentNode, "marginLeft", "40px");
						}
						else
						{
							if (subTreeNode.getParent().getParent().id == "dijit__TreeNode_0" && (j > 0))
							{
								dojo.style(subTreeNode.rowNode.parentNode, "marginLeft", "40px");
							}
						}
						
						//do not expand if the the same Task is shown in the tree.
						if (treeNode.getParent().item.cvid == subTreeNode.item.cvid)
						{
							subTreeNode.isExpandable = false;
							this._removeAndAddClass(subTreeNode.expandoNode, ["dijitTreeExpando", "dijitTreeExpandoLeaf"]);					
						}
						
						foundResults = true;
					}
					else
					{
						var subTreeNode = new dijit._TreeNode(
						{
							item: relationResults[j],
							tree: treeNode.tree,
							isExpandable: true,
							label: relationResults[j].__title__
						});

						treeNode.addChild(subTreeNode);
						
						if (!this.isIe6 && (subTreeNode.getParent().getParent().id == "dijit__TreeNode_0"))
						{
							dojo.style(subTreeNode.rowNode.parentNode, "marginLeft", "40px");
						}
						
						foundResults = true;
						this._removeAndAddClass(subTreeNode.expandoNode, ["dijitTreeExpando", "dijitTreeExpandoLeaf"]);
					}
				}
			}
		}		
		
		if (!foundResults)
		{
			// Jsp or wslet tags are not parsed when we do a require on some file.
			var message = window.getMessage();
			
			//display no results found message.
			var subTreeNode = new dijit._TreeNode(
			{
		 		item: {name: "No results found", title: "", noResult: true},
				tree: treeNode.tree,
				isExpandable: false,
				label: message
			});
			
			treeNode.addChild(subTreeNode);
			if (!this.isIe6 && (subTreeNode.getParent().getParent().id == "dijit__TreeNode_0"))
			{
				dojo.style(subTreeNode.rowNode.parentNode, "marginLeft", "40px");
			}
			
			//'no results found' node will always be last node and is a leaf node.
			this._removeAndAddClass(subTreeNode.expandoNode, ["dijitTreeExpando", "dijitTreeExpandoLeaf"]);			
		}
		
		//if the node is the last node, add css 'dijitTreeIsLast'
		this._removeAndAddClass(treeNode.treeNode.lastChild, ["dijitTreeNode", "dijitTreeIsLast"]);		
	},
	
	/*
	 *  Romove children from a given node.
	 */
	removeChildren: function(nodeWidget)
	{
		var children = nodeWidget.getChildren();
		
		dojo.forEach(children, function(child)
		{
			nodeWidget.removeChild(child);
		});
	}
	
});
